import regress


def test_string_contains_match():
    """
    https://docs.rs/regress/latest/regress/#example-test-if-a-string-contains-a-match
    """
    regex = regress.Regex(r"\d{4}")
    assert regex.find("2020-20-05") is not None


def test_iterating_over_matches():
    """
    https://docs.rs/regress/latest/regress/#example-iterating-over-matches
    """

    regex = regress.Regex(r"(\w)\1")
    text = "Frankly, Miss Piggy, I don't give a hoot!"
    result = [text[m.range()] for m in regex.find_iter(text)]
    assert result == ["ss", "gg", "oo"]


def test_capture_groups():
    """
    https://docs.rs/regress/latest/regress/#example-using-capture-groups
    """

    regex = regress.Regex(r"(\d{4})")
    text = "Today is 2020-20-05"

    match = regex.find(text)
    assert match is not None

    group = match.group(1)
    assert group

    assert text[group] == "2020"


def test_named_group():
    regex = regress.Regex(r"(?<animal>fox|dog)")

    text = "The quick brown fox jumped over the lazy dog's back"

    match = regex.find(text)
    assert match is not None

    group = match.named_group("animal")
    assert group

    assert text[group] == "fox"


def test_error_handling():
    try:
        regress.Regex(r"(")
    except regress.RegressError:
        pass
    else:
        assert False, "error not reached"



from pathlib import Path
from tempfile import TemporaryDirectory
import os

import nox

ROOT = Path(__file__).parent
TESTS = ROOT / "tests"
PYPROJECT = ROOT / "pyproject.toml"

SUPPORTED = ["3.8", "3.9", "3.10", "pypy3.10", "3.11", "3.12"]
LATEST = SUPPORTED[-1]

nox.options.default_venv_backend = "uv|virtualenv"
nox.options.sessions = []


def session(default=True, python=LATEST, **kwargs):  # noqa: D103
    def _session(fn):
        if default:
            nox.options.sessions.append(kwargs.get("name", fn.__name__))
        return nox.session(python=python, **kwargs)(fn)

    return _session


@session(python=SUPPORTED)
def tests(session):
    """
    Run the test suite with a corresponding Python version.
    """
    session.install(ROOT, "-r", TESTS / "requirements.txt")

    if session.posargs and session.posargs[0] == "coverage":
        if len(session.posargs) > 1 and session.posargs[1] == "github":
            github = os.environ["GITHUB_STEP_SUMMARY"]
        else:
            github = None

        session.install("coverage[toml]")
        session.run("coverage", "run", "-m", "pytest", TESTS)
        if github is None:
            session.run("coverage", "report")
        else:
            with open(github, "a") as summary:
                summary.write("### Coverage\n\n")
                summary.flush()  # without a flush, output seems out of order.
                session.run(
                    "coverage",
                    "report",
                    "--format=markdown",
                    stdout=summary,
                )
    else:
        session.run("pytest", *session.posargs, TESTS)


@session(tags=["build"])
def build(session):
    """
    Build a distribution suitable for PyPI and check its validity.
    """
    session.install("build", "twine")
    with TemporaryDirectory() as tmpdir:
        session.run("python", "-m", "build", ROOT, "--outdir", tmpdir)
        session.run("twine", "check", "--strict", tmpdir + "/*")
